package rfsm

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Rfsm.CompilerSource

The command-line compiler

Sourcemodule type T = sig ... end

Output signature of the functor Compiler.Make

Sourcemodule type PARSER = sig ... end

Signature for the Parser input to the functor Compiler.Make

Sourcemodule type LEXER = sig ... end

Signature for the Lexer input to the functor Compiler.Make

Functor building the compiler structure given a language definition, a lexer and a parser implementation