package rfsm

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file subst.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
(**********************************************************************)
(*                                                                    *)
(*              This file is part of the RFSM package                 *)
(*                                                                    *)
(*  Copyright (c) 2018-present, Jocelyn SEROT.  All rights reserved.  *)
(*                                                                    *)
(*  This source code is licensed under the license found in the       *)
(*  LICENSE file in the root directory of this source tree.           *)
(*                                                                    *)
(**********************************************************************)

type 'a t = (Ident.t * 'a) list

let apply phi id = List.assoc id phi

let pp pp_v fmt phi = Ext.List.pp_assoc (Ident.pp,pp_v) fmt phi (** For debug only *)